@@ -188,7 +188,7 @@ class U6(Device):
     def __init__(self, debug = False, autoOpen = True, **kargs):
         """
         Name: U6.__init__(self, debug = False, autoOpen = True, **kargs)
-        Args: debug, Do you want debug information?
+        Args: debug is False, True (for stdout) or a logging.Logger
               autoOpen, If true, then the constructor will call open for you
               **kargs, The arguments to be passed to open.
         Desc: Your basic constructor.
@@ -686,6 +686,10 @@ class U6(Device):
         self.streamSamplesPerPacket = SamplesPerPacket
         self.streamChannelNumbers = ChannelNumbers
         self.streamChannelOptions = ChannelOptions
+
+        # Get a count of duplicate channels.
+        self.streamChannelDuplicates = collections.Counter({key:cnt for key, cnt in collections.Counter(self.streamChannelNumbers).items() if cnt > 1})
+
         self.streamConfiged = True
 
         if InternalStreamClockFrequency == 1:
@@ -723,27 +727,39 @@ class U6(Device):
         returnDict = collections.defaultdict(list)
 
         numChannels = len(self.streamChannelNumbers)
-        j = self.streamPacketOffset
-        for packet in self.breakupPackets(result, numBytes):
-            for sample in self.samplesFromPacket(packet):
-                if j >= numChannels:
-                    j = 0
 
-                if self.streamChannelNumbers[j] in (193, 194):
-                    value = unpack('<BB', sample)
-                elif self.streamChannelNumbers[j] >= 200:
-                    value = unpack('<H', sample)[0]
-                else:
-                    value = unpack('<H', sample)[0]
-                    gainIndex = (self.streamChannelOptions[j] >> 4) & 0x3
-                    value = self.binaryToCalibratedAnalogVoltage(gainIndex, value, is16Bits = True, resolutionIndex = 1)
+        if self.streamPacketOffset >= numChannels:
+            self.streamPacketOffset = 0
 
-                returnDict["AIN%s" % self.streamChannelNumbers[j]].append(value)
+        all_samples = [x for packet in self.breakupPackets(result, numBytes) for x in self.samplesFromPacket(packet)]
+        for i in range(min(len(all_samples), numChannels)):
+            packed_values = all_samples[i::numChannels]
+            channelIndex = (self.streamPacketOffset + i) % numChannels
 
-                j += 1
+            if self.streamChannelNumbers[channelIndex] in (193, 194):
+                values = [unpack('<BB', sample) for sample in packed_values]
+            elif self.streamChannelNumbers[channelIndex] >= 200:
+                values = [unpack('<H', sample)[0] for sample in packed_values]
+            else:
+                values = [unpack('<H', sample)[0] for sample in packed_values]
+                gainIndex = (self.streamChannelOptions[channelIndex] >> 4) & 0x3
+                values = self.binaryListToCalibratedAnalogVoltages(gainIndex, values, is16Bits=True, resolutionIndex=1)
 
-            self.streamPacketOffset = j
+            if self.streamChannelNumbers[channelIndex] in self.streamChannelDuplicates:
+                # Store duplicate channel's data temporarily in a list of list.
+                returnDict["AIN%s" % self.streamChannelNumbers[channelIndex]].append(values)
+            else:
+                returnDict["AIN%s" % self.streamChannelNumbers[channelIndex]] = values
 
+        # Handle duplicate channels and interleave their data into one list.
+        for key, cnt in self.streamChannelDuplicates.items():
+            if "AIN%s" % key in returnDict:
+                newList = [None]*sum([len(li) for li in returnDict["AIN%s" % key]])
+                for i in range(len(returnDict["AIN%s" % key])):
+                    newList[i::cnt] = returnDict["AIN%s" % key][i]
+                returnDict["AIN%s" % key] = newList
+
+        self.streamPacketOffset = (self.streamPacketOffset + len(all_samples)) % numChannels
         return returnDict
 
     def watchdog(self, Write = False, ResetOnTimeout = False, SetDIOStateOnTimeout = False, TimeoutPeriod = 60, DIOState = 0, DIONumber = 0):
@@ -851,7 +867,6 @@ class U6(Device):
 
         oddPacket = False
         if numSPIBytes%2 != 0:
-            SPIBytes.append(0)
             numSPIBytes = numSPIBytes + 1
             oddPacket = True
 
@@ -886,7 +901,7 @@ class U6(Device):
         if oddPacket:
             command[13] = numSPIBytes - 1
 
-        command[14:] = SPIBytes
+        command[14:] = SPIBytes + [0]*oddPacket
 
         result = self._writeRead(command, 8+numSPIBytes, [0xF8, 1+(numSPIBytes//2), 0x3A])
 
@@ -977,7 +992,6 @@ class U6(Device):
         oddPacket = False
         if numBytes % 2 != 0:
             oddPacket = True
-            AsynchBytes.append(0)
             numBytes = numBytes + 1
 
         command = [0] * (8 + numBytes)
@@ -991,7 +1005,7 @@ class U6(Device):
         command[7] = numBytes
         if oddPacket:
             command[7] = numBytes - 1
-        command[8:] = AsynchBytes
+        command[8:] = AsynchBytes + [0]*oddPacket
 
         result = self._writeRead(command, 10, [0xF8, 0x02, 0x15])
 
@@ -1063,7 +1077,6 @@ class U6(Device):
         oddPacket = False
         if numBytes % 2 != 0:
             oddPacket = True
-            I2CBytes.append(0)
             numBytes = numBytes + 1
 
         command = [0] * (14+numBytes)
@@ -1093,7 +1106,7 @@ class U6(Device):
         if oddPacket:
             command[12] = numBytes - 1
         command[13] = NumI2CBytesToReceive
-        command[14:] = I2CBytes
+        command[14:] = I2CBytes + [0]*oddPacket
 
         oddResponse = False
         if NumI2CBytesToReceive % 2 != 0:
@@ -1189,8 +1202,7 @@ class U6(Device):
         >>> myU6.calInfo
         <ainDiffOffset: -2.46886488446,...>
         """
-        if self.debug is True:
-            print("Calibration data retrieval")
+        self._debugprint("Calibration data retrieval")
 
         self.calInfo.nominal = False
 
@@ -1298,37 +1310,65 @@ class U6(Device):
             self.calInfo.proAinNegSlope = [self.calInfo.proAin10vNegSlope, self.calInfo.proAin1vNegSlope, self.calInfo.proAin100mvNegSlope, self.calInfo.proAin10mvNegSlope]
             self.calInfo.proAinCenter = [self.calInfo.proAin10vCenter, self.calInfo.proAin1vCenter, self.calInfo.proAin100mvCenter, self.calInfo.proAin10mvCenter]
 
+    def getCalibratedSlopesCenter(self, gainIndex, resolutionIndex):
+        """
+        Name: U6.getCalibratedSlopesCenter(gainIndex,
+                                           resolutionIndex)
+
+        Desc: Get the slopes and center for converting a raw ADC voltage
+              into a calibrated voltage.
+        """
+        if self.isPro and (resolutionIndex > 8 or resolutionIndex == 0):
+            # Use hi-res calibration constants
+            center = self.calInfo.proAinCenter[gainIndex]
+            negSlope = self.calInfo.proAinNegSlope[gainIndex]
+            posSlope = self.calInfo.proAinSlope[gainIndex]
+        else:
+            # Use normal calibration constants
+            center = self.calInfo.ainCenter[gainIndex]
+            negSlope = self.calInfo.ainNegSlope[gainIndex]
+            posSlope = self.calInfo.ainSlope[gainIndex]
+
+        return negSlope, posSlope, center
+
     def binaryToCalibratedAnalogVoltage(self, gainIndex, bytesVoltage, is16Bits=False, resolutionIndex=0):
         """
         Name: U6.binaryToCalibratedAnalogVoltage(gainIndex, bytesVoltage, 
                                                  is16Bits = False, resolutionIndex = 0)
         Args: gainIndex, which gain index did you use?
-              bytesVoltage, bytes returned from the U6
-              is16Bits, set to True if bytesVoltage is 16 bits (not 24)
+              bytesVoltage, voltage bytes returned from the U6.
+              is16Bits, set to True if bytesVoltage is 16-bits (not 24).
               resolutionIndex, which resolution index did you use?  Set this for
                                U6-Pro devices to ensure proper hi-res conversion.
-        Desc: Converts binary voltage to an analog value.
+        Desc: Converts the binary voltage to a calibrated analog voltage value.
         """
+        negSlope, posSlope, center = self.getCalibratedSlopesCenter(gainIndex, resolutionIndex)
+
         if not is16Bits:
-            bits = float(bytesVoltage)/256
-        else:
-            bits = float(bytesVoltage)
+            bytesVoltage = bytesVoltage / 256.0
 
-        if self.isPro and (resolutionIndex > 8 or resolutionIndex == 0):
-            #Use hi-res calibration constants
-            center = self.calInfo.proAinCenter[gainIndex]
-            negSlope = self.calInfo.proAinNegSlope[gainIndex]
-            posSlope = self.calInfo.proAinSlope[gainIndex]
+        if bytesVoltage < center:
+            return (center - bytesVoltage) * negSlope
         else:
-            #Use normal calibration constants
-            center = self.calInfo.ainCenter[gainIndex]
-            negSlope = self.calInfo.ainNegSlope[gainIndex]
-            posSlope = self.calInfo.ainSlope[gainIndex]
+            return (bytesVoltage - center) * posSlope
 
-        if bits < center:
-            return (center - bits) * negSlope
+    def binaryListToCalibratedAnalogVoltages(self, gainIndex, bytesVoltageList, is16Bits=False, resolutionIndex=0):
+        """
+        Name: U6.binaryToCalibratedAnalogVoltage(gainIndex, bytesVoltage, 
+                                                 is16Bits = False, resolutionIndex = 0)
+        Args: gainIndex, which gain index did you use?
+              bytesVoltageList, list of voltage bytes returned from the U6.
+              is16Bits, set to True if bytesVoltage is 16-bits (not 24).
+              resolutionIndex, which resolution index did you use?  Set this for
+                               U6-Pro devices to ensure proper hi-res conversion.
+        Desc: Converts a list of binary voltages to a list of calibrated analog
+              voltage values.
+        """
+        negSlope, posSlope, center = self.getCalibratedSlopesCenter(gainIndex, resolutionIndex)
+        if not is16Bits:
+            return [(center - value/256.0) * negSlope if value < center else (value/256.0 - center) * posSlope for value in bytesVoltageList]
         else:
-            return (bits - center) * posSlope
+            return [(center - value) * negSlope if value < center else (value - center) * posSlope for value in bytesVoltageList]
 
     def binaryToCalibratedAnalogTemperature(self, bytesTemperature, is16Bits=False):
         """
