@@ -76,7 +76,7 @@ class UE9(Device):
     def __init__(self, debug = False, autoOpen = True, **kargs):
         """
         Name: UE9.__init__(self)
-        Args: debug, True for debug information
+        Args: debug is False, True (for stdout) or a logging.Logger
         Desc: Your basic constructor.
         
         >>> myUe9 = ue9.UE9()
@@ -878,7 +878,7 @@ class UE9(Device):
         Args: None
         Desc: Clears the streaming data on the stream USB endpoint or TCP socket
               that was left over from a previous stream.
-        Note: Use before and/or after streaming.  Timeout delay can occur occur.
+        Note: Use before and/or after streaming.  Timeout delay can occur.
         """
         try:
             for i in range(10):
@@ -1121,8 +1121,7 @@ class UE9(Device):
                 e = streamByteToInt(result[11+offset])
                 if e != 0:
                     errors += 1
-                    if self.debug:
-                        print(e)
+                    self._debugprint(e)
                 i+=1
 
             if len(result) == 0  and self.ethernet == False:
@@ -1324,7 +1323,6 @@ class UE9(Device):
 
         oddPacket = False
         if numSPIBytes%2 != 0:
-            SPIBytes.append(0)
             numSPIBytes = numSPIBytes + 1
             oddPacket = True
 
@@ -1358,7 +1356,7 @@ class UE9(Device):
         if oddPacket:
             command[13] = numSPIBytes - 1
 
-        command[14:] = SPIBytes
+        command[14:] = SPIBytes + [0]*oddPacket
 
         result = self._writeRead(command, 8+numSPIBytes, [0xF8, 1+(numSPIBytes//2), 0x3A])
 
@@ -1442,7 +1440,6 @@ class UE9(Device):
 
         oddPacket = False
         if numBytes % 2 != 0:
-            AsynchBytes.append(0)
             numBytes = numBytes + 1
             oddPacket = True
 
@@ -1459,7 +1456,7 @@ class UE9(Device):
         if oddPacket:
             command[7] = numBytes - 1
 
-        command[8:] = AsynchBytes
+        command[8:] = AsynchBytes + [0]*oddPacket
 
         result = self._writeRead(command, 10, [0xF8, 0x02, 0x15])
 
@@ -1522,7 +1519,6 @@ class UE9(Device):
 
         oddPacket = False
         if numBytes % 2 != 0:
-            I2CBytes.append(0)
             numBytes = numBytes + 1
             oddPacket = True
 
@@ -1552,7 +1548,7 @@ class UE9(Device):
         if oddPacket:
             command[12] = numBytes - 1
         command[13] = NumI2CBytesToReceive
-        command[14:] = I2CBytes
+        command[14:] = I2CBytes + [0]*oddPacket
 
         oddResponse = False
         if NumI2CBytesToReceive % 2 != 0:
